@@ -1,4 +1,5 @@
 import os, pickle
+from tempfile import mktemp
 
 from pickle_warehouse.identifiers import parse as parse_identifier
 
@@ -14,6 +15,12 @@ except NameError:
 else:
     OpenError = FileNotFoundError
 
+try:
+    PermissionError
+except NameError:
+    class PermissionError(EnvironmentError):
+        pass
+
 try:
     FileNotFoundError
 except NameError:
@@ -55,17 +62,30 @@ class Warehouse:
         if (not self.mutable) and os.path.exists(fn):
             raise PermissionError('This warehouse is immutable, and %s already exists.' % fn)
         else:
-            with open(fn, 'wb') as fp:
+            tmp = mktemp()
+            with open(tmp, 'wb') as fp:
                 self.serializer.dump(obj, fp)
+            os.rename(tmp, fn)
 
     def __getitem__(self, index):
+        fn = self.filename(index)
+
         try:
-            with open(self.filename(index), 'rb') as fp:
+            mtime_before = os.path.getmtime(fn)
+        except OSError:
+            mtime_before = None
+
+        try:
+            with open(fn, 'rb') as fp:
                 item = self.serializer.load(fp)
         except OpenError as e:
             raise KeyError(*e.args)
         else:
-            return item
+            mtime_after = os.path.getmtime(fn)
+            if mtime_before == mtime_after:
+                return item
+            else:
+                raise EnvironmentError('File was edited during read: %s' % fn)
 
     def __delitem__(self, index):
         if not self.mutable:
